/*
 * setup.c Copyright Jan Panteltje 2005 and later.
 *
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "tinyptc.h"

#include "xmars3d.h"

int save_setting(char *type, char *string)
{
char pathfilename[TEMP_SIZE];
FILE *setting_file;
char temp[TEMP_SIZE];

if(debug_flag)
	{
	fprintf(stderr, "save_setting(): arg type=%s string=%s\n", type, string);
	}

if(! type) return 0;
if(! string) return 0;

/* test of filename exists, ask for overwrite */
if(strcmp(type, "last_settings") != 0)
	{
	sprintf(pathfilename, "%s/.xmars3d/setup/%s", home_dir, type);
	setting_file = fopen(pathfilename, "r");
	if(setting_file)
		{
		fclose(setting_file);
		sprintf(temp, "file %s already exists, overwrite (y/n)? ", pathfilename);

		if(debug_flag)
			{
			fprintf(stderr, "%s", temp);
			}

		text_to_x11(\
		temp, 70, 160, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);

		while(1)
			{
			ptc_update(display_data1);
			set_lcd(0);

			if(key_press == 'y') break;
			if(key_press == 'n')
				{
				key_press = 0;

				return 0;
				}

			usleep(20000);
			}

		key_press = 0;
		} /* end if file exists */
	} /* end if save as mode */

/* combine with home directory */
sprintf(pathfilename, "%s/.xmars3d/setup/%s", home_dir, type);
setting_file = fopen(pathfilename, "w");
if(! setting_file)
	{
	sprintf(temp, "save_settings(): cannot open file %s for write", pathfilename);

	if(debug_flag)
		{
		fprintf(stderr, "%s\n", temp);
		}

	temp[400] = 0;

	text_to_x11(\
		temp, 10, 160, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);

	return 0;
	}

fprintf(setting_file, "%s", string);
fclose(setting_file);
return 1;	
}/* end function save_setting */


char *load_setting(char *type)
{
int a;
char temp[TEMP_SIZE];
char pathfilename[TEMP_SIZE];
FILE *setting_file;
int line_len;
char *ptr;

if(debug_flag)
	{
	fprintf(stderr, "load_setting(): arg type=%s\n", type);
	}

if(! type) return 0;

/* combine with home directory */
sprintf(pathfilename, "%s/.xmars3d/setup/%s", home_dir, type);
setting_file = fopen(pathfilename, "r");
if(! setting_file)
	{
	sprintf(temp, "load_settings(): cannot open file %s for read", pathfilename);

	if(debug_flag)
		{
		fprintf(stderr, "%s\n", temp);
		}
//fprintf(stderr, "display_data1=%p\n", display_data1);

	if(strcmp(type, "last_settings") != 0)
		{
		text_to_x11(\
		temp, 70, 160, display_width, display_height, 0xff, 0xff, 0xff, 0x10 , 0x10, 0x10, display_data1);

		while(1)
			{
			ptc_update(display_data1);
			set_lcd(0);

			if(key_press != 0) return 0;

			usleep(20000);
			}

		key_press = 0;
		}

	return 0;
	}

//ssize_t getline (char **LINEPTR, size_t *N, FILE *STREAM)
ptr = NULL;
a = getline(&ptr, &line_len, setting_file); 

//fprintf(stderr, "a=%d line_len=%d\n", a, line_len);

fclose(setting_file);
return ptr;
}/* end function load_setting */



int save_general_settings(char *filename)
{
char temp[TEMP_SIZE];

if(debug_flag)
	{
	fprintf(stderr, "save_general_settings(): arg filename=%s\n", filename);	
	}

if(! filename) return 0;

sprintf(temp,\
"%d %d %d %d %d %d %d %d %d %.2f %d %d %s %s %d",\
xpos,\
ypos,\
display_width,\
display_height,\
delta_x,\
delta_y,\
delta_z,\
xshift,\
yshift,\
gamma_factor,\
zoom_factor,\
help_flag,\
filename_left,\
filename_right,\
enlarge_factor\
);

if(!save_setting(filename, temp) ) return 0;

return 1;
}/* end function save_general_settings */


int load_general_settings(char *filename)
{
int error;
char *ptr;

if(debug_flag)
	{
	fprintf(stdout, "load_general_settings(): arg filename=%s\n", filename);
	}

if(! filename) return 0;

error = 0;
ptr = load_setting(filename);

if(! ptr) return 0;

if(filename_left) free(filename_left);
if(filename_right) free(filename_right);

/* for compatibility loading files made by older versions then 0.3.9 */
enlarge_factor = 1;
if(ptr)
	{
	sscanf(ptr,\
	"%d %d %d %d %d %d %d %d %d %lf %d %d %as %as %d",\
	&xpos,\
	&ypos,\
	&display_width,\
	&display_height,\
	&delta_x,\
	&delta_y,\
	&delta_z,\
	&xshift,\
	&yshift,\
	&gamma_factor,\
	&zoom_factor,\
	&help_flag,\
	&filename_left,\
	&filename_right,\
	&enlarge_factor\
	);

	free(ptr);
	}
else
	{
	error = 1;
	}

if(debug_flag)
	{
	fprintf(stderr, "load_general_settings(): filename_left=%s filename_right=%s\n",\
	filename_left, filename_right);
	}

if(strcmp(filename_right, "(null)") == 0)
	{
	free(filename_right);
	
	filename_right = 0;
	}

/* 
create setup files from defaults if some not found,
else use values just read.
*/
if(error) 
	{
	if(strcmp(filename, "last_settings") == 0)
		{
		save_general_settings(filename);

		return 1;
		}
	else
		{
		return(0);
		}
	}

return 1;
} /* end function load_general_settings */


